﻿using System;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using FishbowlConnect.DAL;
using FishbowlConnect.Helpers;
using FishbowlConnect.Helpers.Conversions;
using FishbowlConnect.Helpers.IO;

namespace FishbowlConnect
{
    public class FishbowlSession : IDisposable
    {
        private Socket _socket;
        private NetworkStream _networkStream;
        private EndianBinaryWriter _endianBinaryWriter;
        private EndianBinaryReader _endianBinaryReader;
        private IPEndPoint fishbowlServer { get; set; }

        /// <summary>
        /// Fishbowl User Name
        /// </summary>
        public string UserName { get; set; }

        /// <summary>
        /// Fishbowl Unencrypted User Password
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// Fishbowl Authentication Key
        /// </summary>
        public string SessionKey { get; private set; }

        /// <summary>
        /// Session Connection State ( Used 1 seat of your license )
        /// </summary>
        public bool IsConnected { get; private set; }

        /// <summary>
        /// Session Authentication State
        /// </summary>
        public bool IsAuthenticated { get; private set; }

        /// <summary>
        /// Fishbowl Inventory Helper
        /// </summary>
        /// <param name="fishbowlServerAddress">IP Address of Server (e.g. 10.36.2.136)</param>
        public FishbowlSession(string fishbowlServerAddress)
            : this(IPAddress.Parse(fishbowlServerAddress))
        {
        }

        /// <summary>
        /// Fishbowl Inventory Helper
        /// </summary>
        /// <param name="fishbowlServerAddress">IPAddress of Server</param>
        /// <param name="tcpPort">TCP/IP Port (Default set to 28192)</param>
        public FishbowlSession(IPAddress fishbowlServerAddress, int tcpPort = 28192)
        {
            fishbowlServer = new IPEndPoint(fishbowlServerAddress, tcpPort);
        }

        /// <summary>
        /// Open Connection to Fishbowl Server and Login
        /// ** Connection will increase Fishbowl user count that is limited by fishbowl license key.
        /// </summary>
        /// <param name="userName">Fishbowl User Name</param>
        /// <param name="password">Fishbowl Unencrypted User Password</param>
        public void Connect(string userName, string password)
        {
            if (!Utilities.VerifyIPOnline(fishbowlServer.Address.ToString(), 28192, 1000))
            {
                throw new Exception("Unable to communicate with destination IP address.");
            }

            _socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            _socket.Connect(fishbowlServer);

            _networkStream = new NetworkStream(_socket);
            _endianBinaryWriter = new EndianBinaryWriter(EndianBitConverter.Big, _networkStream);
            _endianBinaryReader = new EndianBinaryReader(EndianBitConverter.Big, _networkStream);

            if (_socket.Connected) IsConnected = true;

            Login(userName, password);
        }

        /// <summary>
        /// Open Connection to Fishbowl Server
        /// ** Connection will increase Fishbowl user count that is limited by fishbowl license key.
        /// </summary>
        public void Connect()
        {
            if (!Utilities.VerifyIPOnline(fishbowlServer.Address.ToString(), 28192, 1000))
            {
                throw new Exception("Unable to communicate with destination IP address.");
            }

            _socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            _socket.Connect(fishbowlServer);

            _networkStream = new NetworkStream(_socket);
            _endianBinaryWriter = new EndianBinaryWriter(EndianBitConverter.Big, _networkStream);
            _endianBinaryReader = new EndianBinaryReader(EndianBitConverter.Big, _networkStream);

            if (_socket.Connected) IsConnected = true;
        }

        /// <summary>
        /// Login to Fishbowl and obtain a session key to make data requests.
        /// </summary>
        /// <param name="userName">Fishbowl User Name</param>
        /// <param name="password">Fishbowl Unencrypted User Password</param>
        /// <returns>Login Status Code</returns>
        public string Login(string userName, string password)
        {
            UserName = userName;
            Password = password;

            return Login();
        }

        /// <summary>
        /// Login to Fishbowl and obtain a session key to make data requests.
        /// </summary>
        /// <returns>Login Status Code</returns>
        public string Login()
        {
            if (String.IsNullOrEmpty(UserName))
            {
                throw new Exception("UserName was not assigned.");
            }

            if (String.IsNullOrEmpty(Password))
            {
                throw new Exception("Password was not assigned.");
            }

            string resultCode = "9999";

            // MD5 Base64 Encode Password
            string md5HashString = Utilities.EncryptPassword(Password);

            // Build Login Request
            FbiXml request = new FbiXml();
            request.Ticket = new Ticket();
            LoginRqType[] loginRqType = new LoginRqType[1];
            loginRqType[0] = new LoginRqType { IADescription = "Used to lookup inventory status on a Fishbowl part.", IAID = "00010", IAName = "eServices Stock Check", UserName = UserName, UserPassword = md5HashString };
            FbiMsgsRq fbiMsgsRq = new FbiMsgsRq { Items = loginRqType };
            request.Item = fbiMsgsRq;

            ASCIIEncoding encoding = new ASCIIEncoding();

            byte[] bytes = encoding.GetBytes(SerializeToXMLString(request));

            _endianBinaryWriter.Write(bytes.Length);
            _endianBinaryWriter.Write(bytes);
            _endianBinaryWriter.Flush();

            int i = _endianBinaryReader.ReadInt32();
            byte[] respBytes = new byte[i];
            _endianBinaryReader.Read(respBytes, 0, i);
            string output = encoding.GetString(respBytes, 0, i);

            // Deserialize XML Response to Object
            FbiXml response = DeserializeFromXMLString<FbiXml>(output);
            FbiMsgsRs fbiMsgRs = (FbiMsgsRs)response.Item;
            
            if (fbiMsgRs.statusCode == "1000")
            {
                LoginRsType loginRsType = (LoginRsType)fbiMsgRs.Items[0];

                // Check for success
                if (loginRsType.statusCode == "1000")
                {
                    SessionKey = response.Ticket.Key;
                    IsAuthenticated = true;
                }
                else
                {
                    resultCode = loginRsType.statusCode;
                }
            }
            else
            {
                resultCode = fbiMsgRs.statusCode;
            }

            return resultCode;
        }

        /// <summary>
        /// Fishbowl Request Helper
        /// Wraps up the sending and receiving of data, requires the object to be connected and authenticated.
        /// </summary>
        /// <param name="xmlRequest">FbiXML formatted Fishbowl Request</param>
        /// <returns>XML Response Message</returns>
        public string IssueRequest(string xmlRequest)
        {
            if (IsConnected != true)
            {
                throw new Exception("Connection Error, you are not connected to the fishbowl server.");
            }

            if (IsAuthenticated != true)
            {
                throw new Exception("Authentication Error, you are not authenticated with the fishbowl server.");
            }

            ASCIIEncoding encoding = new ASCIIEncoding();

            byte[] bytes = encoding.GetBytes(xmlRequest);

            _endianBinaryWriter.Write(bytes.Length);
            _endianBinaryWriter.Write(bytes);
            _endianBinaryWriter.Flush();

            int i = _endianBinaryReader.ReadInt32();
            byte[] respBytes = new byte[i];
            _endianBinaryReader.Read(respBytes, 0, i);
            return encoding.GetString(respBytes, 0, i);
        }

        /// <summary>
        /// Fishbowl Request Helper
        /// Wraps up the sending and receiving of data, requires the object to be connected and authenticated.
        /// </summary>
        /// <param name="requestObject">Fishbowl Request Object Array</param>
        /// <param name="statusCode">Fishbowl Matching Response to Request Array</param>
        /// <returns>XML Response Message</returns>
        public T IssueRequest<T>(object requestObject)
        {
            object[] requestObjects = new object[1];
            requestObjects[0] = requestObject;

            FbiXml request = new FbiXml();
            request.Ticket = new Ticket { Key = SessionKey, UserID = "85" };
            FbiMsgsRq fbiMsgsRq = new FbiMsgsRq { Items = requestObjects };
            request.Item = fbiMsgsRq;

            string xmlRequest = SerializeToXMLString(request);

            string xmlResponse = IssueRequest(xmlRequest);

            FbiXml response = DeserializeFromXMLString<FbiXml>(xmlResponse);

            FbiMsgsRs fbiMsgsRs = (FbiMsgsRs)response.Item;

            string statusCode = fbiMsgsRs.statusCode;

            if (statusCode != "1000")
            {
                return default(T);
            }

            T tagResponse = (T)fbiMsgsRs.Items[0];

            return tagResponse;
        }

        /// <summary>
        /// Fishbowl Request Helper
        /// Wraps up the sending and receiving of data, requires the object to be connected and authenticated.
        /// </summary>
        /// <param name="requestObjects">Fishbowl Request Object Array</param>
        /// <param name="statusCode">Fishbowl Matching Response to Request Array</param>
        /// <returns>XML Response Message</returns>
        public T IssueRequest<T>(object[] requestObjects)
        {
            if (requestObjects.Length == 0 || requestObjects[0] == null)
            {
                throw new ArgumentException("Invalid array of request objects, you array is empty or contains null values.");
            }

            FbiXml request = new FbiXml();
            request.Ticket = new Ticket { Key = SessionKey, UserID = "85" };
            FbiMsgsRq fbiMsgsRq = new FbiMsgsRq { Items = requestObjects };
            request.Item = fbiMsgsRq;

            string xmlRequest = SerializeToXMLString(request);

            string xmlResponse = IssueRequest(xmlRequest);

            FbiXml response = DeserializeFromXMLString<FbiXml>(xmlResponse);

            FbiMsgsRs fbiMsgsRs = (FbiMsgsRs)response.Item;

            string statusCode = fbiMsgsRs.statusCode;

            if (statusCode != "1000")
            {
                return default(T);
            }

            T tagResponse = (T)fbiMsgsRs.Items[0];

            return tagResponse;
        }

        /// <summary>
        /// Get Fishbowl Inventory
        /// </summary>
        /// <param name="part">Fishbowl Part Number</param>
        /// <param name="exceptionMessage">Fishbowl Lookup Message</param>
        /// <returns>Inventory Quantity</returns>
        public int GetInventory(string part, out string exceptionMessage)
        {
            // Set default return values
            exceptionMessage = string.Empty;
            int available = 0;

            // Build PartTagQueryRqType Request
            PartTagQueryRqType partTagQueryRqType = new PartTagQueryRqType { LocationGroup = "Product", PartNum = part };

            // Submit Request get Response
            PartTagQueryRsType partTagQueryRsType = IssueRequest<PartTagQueryRsType>(partTagQueryRqType);

            // Unregister Part, show custom message
            if (partTagQueryRsType.statusCode != "1000")
            {
                exceptionMessage = Utilities.StatusCodeMessage(partTagQueryRsType.statusCode);
            }

            // Tag object is optional, if there is no tag element you can not get the quantity
            if (partTagQueryRsType.Tag != null && partTagQueryRsType.Tag.Length > 0)
            {
                Tag tag = partTagQueryRsType.Tag[0];
                available = int.Parse(tag.Quantity);
            }

            // Stock Quantity
            return available;
        }

        /// <summary>
        /// Close Connection to Fishbowl
        /// ** Closing connection will decrease Fishbowl user count that is limited by fishbowl license key.
        /// </summary>
        public void Disconnect()
        {
            if (_socket != null && _socket.Connected)
                _socket.Shutdown(SocketShutdown.Both);
            _socket.Disconnect(false);

            IsConnected = false;
            IsAuthenticated = false;
        }

        /// <summary>
        /// Implements IDisposable
        /// </summary>
        public void Dispose()
        {
            Disconnect();

            if (_socket != null)
                _socket.Dispose();

            if (_networkStream != null)
                _networkStream.Dispose();

            if (_endianBinaryReader != null)
                _endianBinaryReader.Dispose();

            if (_endianBinaryWriter != null)
                _endianBinaryWriter.Dispose();

            if (_socket != null)
                _socket.Dispose();
        }

        /// <summary>
        /// Deserialize XML String to Strongly Typed Object
        /// </summary>
        /// <typeparam name="T">Object Type</typeparam>
        /// <param name="xml">Serialized XML Object</param>
        /// <returns></returns>
        public static T DeserializeFromXMLString<T>(string xml)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(xml);
            MemoryStream mem = new MemoryStream(bytes);
            XmlSerializer ser = new XmlSerializer(typeof(T));
            return (T)ser.Deserialize(mem);
        }

        /// <summary>
        /// Serialize Object to RAW XML String
        /// ** No namespace, XML declaration, etc..
        /// </summary>
        /// <param name="objectToSerialize">Object to Serialize to XML</param>
        /// <returns>Serialized XML Object</returns>
        private static string SerializeToXMLString(object objectToSerialize)
        {
            MemoryStream mem = new MemoryStream();

            XmlSerializer ser = new XmlSerializer(objectToSerialize.GetType());
            XmlSerializerNamespaces ns = new XmlSerializerNamespaces();
            ns.Add(string.Empty, string.Empty);
            XmlWriterSettings writerSettings = new XmlWriterSettings();
            writerSettings.OmitXmlDeclaration = true;
            writerSettings.Indent = true;

            StringWriter stringWriter = new StringWriter();
            using (XmlWriter xmlWriter = XmlWriter.Create(stringWriter, writerSettings))
            {
                ser.Serialize(xmlWriter, objectToSerialize, ns);
            }

            return stringWriter.ToString();
        }
    }
}