﻿using System;
using FishbowlConnect.DAL;
using FishbowlConnect.Helpers;
using NUnit.Framework;

namespace FishbowlConnect.Tests
{
    [TestFixture]
    class FishbowlSessionTests
    {
        private string UserName { get; set; }
        private string Password { get; set; }
        private string PartWithInventory { get; set; }
        private string PartWithoutInventory { get; set; }
        private string PartNotRegistered { get; set; }
        private FishbowlSession Session { get; set; }

        [SetUp]
        public void Init()
        {
            PartWithInventory = "1SR154-400TE25";
            PartWithoutInventory = "1SS355TE-17";
            PartNotRegistered = "MG87J6011-001LAZ0AA";

            UserName = "app";
            Password = "app";

            Session = new FishbowlSession("10.36.2.139");
        }

        [TearDown]
        public void Disponse()
        {
            if (Session.IsConnected)
            {
                Session.Dispose();
            }
        }

        [Test]
        public void CanConnect()
        {
            Session.Connect();
            Assert.IsTrue(Session.IsConnected);
        }

        [Test]
        public void CanConnectAndAuthenticate()
        {
            Session.Connect(UserName, Password);

            Assert.IsTrue(Session.IsConnected);
            Assert.IsTrue(Session.IsAuthenticated);
        }

        [Test]
        public void ReturnError1120_OnBadLogin()
        {
            Session.Connect();

            Session.UserName = "bad_user_name";
            Session.Password = Password;
            string status = Session.Login();

            Assert.IsTrue(Session.IsConnected);
            Assert.IsFalse(Session.IsAuthenticated);
            Assert.AreEqual("1120",status);
        }

        [Test]
        public void ReturnError1120_OnBadPassword()
        {
            Session.Connect();

            Session.UserName = UserName;
            Session.Password = "bad_password";
            string status = Session.Login();

            Assert.IsTrue(Session.IsConnected);
            Assert.IsFalse(Session.IsAuthenticated);
            Assert.AreEqual("1120", status);
        }

        [Test]
        public void CanGenerateFishbowlHashPassword()
        {
            Assert.AreEqual("0qV9wdiD/SH7mVFpnfccxw==", Utilities.EncryptPassword("app"));
        }

        [Test]
        public void CanLogin()
        {
            Session.Connect();

            Session.UserName = UserName;
            Session.Password = Password;
            Session.Login();

            Assert.IsTrue(Session.IsConnected);
            Assert.IsTrue(Session.IsAuthenticated);
        }

        [Test]
        public void CanLoginOverload1()
        {
            Session.Connect();

            Session.Login(UserName, Password);

            Assert.IsTrue(Session.IsConnected);
            Assert.IsTrue(Session.IsAuthenticated);
        }

        [Test]
        public void CanAuthenticateAndGetSessionKey()
        {
            Session.Connect(UserName, Password);

            Assert.IsTrue(Session.IsAuthenticated);
            Assert.IsNotEmpty(Session.SessionKey);
        }

        [Test]
        [ExpectedException(typeof(Exception), ExpectedMessage = "Unable to communicate with destination IP address.", MatchType = MessageMatch.Contains)]
        public void FailConnect_InvalidIPAddress()
        {
            Session = new FishbowlSession("192.168.168.168");
            Session.Connect(UserName, Password);
        }

        [Test]
        [ExpectedException(typeof(ArgumentException), ExpectedMessage = "Invalid array of request objects", MatchType = MessageMatch.Contains)]
        public void FailIssueRequest_ZeroLengthArray()
        {
            PartGetRqType[] partGetRqType = new PartGetRqType[1];
            partGetRqType[0] = null;

            Session.Connect(UserName, Password);

            Session.IssueRequest<PartGetRsType>(partGetRqType);
        }

        [Test]
        [ExpectedException(typeof(InvalidCastException), ExpectedMessage = "Unable to cast object", MatchType = MessageMatch.Contains)]
        public void FailResponse_MisMatchRequestResponseTypes()
        {
            PartGetRqType partGetRqType = new PartGetRqType {Number = "PARTXYZ"};

            Session.Connect(UserName, Password);

            Session.IssueRequest<PartCostRqType>(partGetRqType);
        }

        [Test]
        [ExpectedException(typeof(Exception), ExpectedMessage = "Password", MatchType = MessageMatch.Contains)]
        public void FailLogin_NoPassword()
        {
            Session.UserName = UserName;
            Session.Login();
        }

        [Test]
        [ExpectedException(typeof(Exception), ExpectedMessage = "UserName", MatchType = MessageMatch.Contains)]
        public void FailLogin_NoUsername()
        {
            Session.Password = Password;
            Session.Login();
        }

        [Test]
        [ExpectedException(typeof(Exception), ExpectedMessage = "Connection Error", MatchType = MessageMatch.Contains)]
        public void FailRequest_OnNoConnection()
        {
            string message;
            Session.GetInventory(PartWithInventory, out message);
        }

        [Test]
        [ExpectedException(typeof(Exception), ExpectedMessage = "Authentication Error", MatchType = MessageMatch.Contains)]
        public void FailRequest_OnNoLogin()
        {
            Session.Connect();

            string message;
            Session.GetInventory(PartWithInventory, out message);
        }

        [Test]
        public void PartHasInventory()
        {
            Session.Connect(UserName, Password);

            string message;
            Assert.GreaterOrEqual(Session.GetInventory(PartWithInventory, out message), 0);
        }

        [Test]
        public void PartHasNoInventory()
        {
            Session.Connect(UserName, Password);

            string message;
            int results = Session.GetInventory(PartWithoutInventory, out message);

            Assert.AreEqual(0, results);
        }

        [Test]
        public void PartNotRegisterdInInventory()
        {
            Session.Connect(UserName, Password);

            string message;
            int results = Session.GetInventory(PartNotRegistered, out message);

            Assert.AreEqual(0, results);
            Assert.AreEqual(Utilities.StatusCodeMessage("2000"), message);
        }

        [Test]
        public void CanProcessPartTagQueryRqType()
        {
            Session.Connect(UserName, Password);

            string statusCode;

            PartTagQueryRqType[] partTagQueryRqType = new PartTagQueryRqType[1];
            partTagQueryRqType[0] = new PartTagQueryRqType { LocationGroup = "Product", PartNum = PartWithInventory };

            PartTagQueryRsType partTagQueryRsType = Session.IssueRequest<PartTagQueryRsType>(partTagQueryRqType);

            Assert.AreEqual("1000", partTagQueryRsType.statusCode);
        }

        [Test]
        public void CanProcessPartQueryRqType()
        {
            Session.Connect(UserName, Password);

            string statusCode;

            PartQueryRqType[] partQueryRqTypes = new PartQueryRqType[1];
            partQueryRqTypes[0] = new PartQueryRqType { LocationGroup = "Product", Item = PartWithInventory };

            PartQueryRsType partQueryRsType = Session.IssueRequest<PartQueryRsType>(partQueryRqTypes);

            Assert.AreEqual("1000", partQueryRsType.statusCode);
        }

        [Test]
        public void CanProcessPartGetRqType()
        {
            Session.Connect(UserName, Password);

            string statusCode;

            PartGetRqType[] partGetRqTypes = new PartGetRqType[1];
            partGetRqTypes[0] = new PartGetRqType { Number= PartWithInventory };

            PartGetRsType partGetRsType = Session.IssueRequest<PartGetRsType>(partGetRqTypes);

            Assert.AreEqual("1000", partGetRsType.statusCode);
        }
    }
}
